from selenium import webdriver
from selenium.common.exceptions import WebDriverException
from samuranium.logger import *
from sys import platform


class DriverFactory:

    def __init__(self):
        self.driver = None
        self.multi_driver = []
        self.currentElement = None
        self.listOfElements = []
        self.desired_caps = {}
        self.driver_type = None
        self.implicitly_wait = 5
        self.page_load_timeout = 5
        self.script_load_timeout = 5

    def set_browser_type(self, browser):
        log("Setting Browser: " + browser)
        self.driver_type = browser

    def set_driver(self):
        """This method sets the driver the driver -- Should only be used by go_to_url(browser) method
        :param browser: name of the browser to be created
        """
        if self.driver_type is None:
            self.driver_type = "firefox"
            warning("Browser not defined, using default: 'Firefox'")
            warning("Use set_browser_type(browser_name) to properly set the browser")
        if self.driver is None:
            if self.driver_type.upper() == "firefox".upper():
                self.driver = webdriver.Firefox()
            elif self.driver_type.upper() == "chrome".upper():
                try:
                    self.driver = webdriver.Chrome()
                except WebDriverException as e:
                    fail(e)
            elif self.driver_type.upper() == "iexplorer".upper():
                try:
                    self.driver = webdriver.Ie()
                except WebDriverException as e:
                    fail(e)
            elif self.driver_type.upper() == "edge".upper():
                try:
                    self.driver = webdriver.Edge()
                except WebDriverException as e:
                    fail(e)
            elif self.driver_type.upper() == "safari".upper():
                if platform.upper() is "Darwin".upper():
                    try:
                        self.driver = webdriver.Safari()
                    except WebDriverException as e:
                        fail(e)
                else:
                    fail("Safari browser is only supported on Mac OS")
            elif self.driver_type.upper() == 'AppiumChrome'.upper():
                self.desired_caps = {'platformName': 'Android', 'platformVersion': '4.4',
                                     'deviceName': 'motorola-nexus_6-ZX1G424TK3', 'browserName': 'Chrome',
                                     'app': 'Chrome'}
                log("Opening Driver")
                return webdriver.Remote('http://localhost:4723/wd/hub', self.desired_caps)
            else:
                # ToDo Error implementation
                fail("Browser not defined, the only available values are: 'firefox', 'chrome', 'safari', 'iexplorer' and "
                "'edge'")
                #raise BrowserNotDefined
        self.setup_driver()
        self.multi_driver.append(self.driver)
        return self.driver

    def setup_driver(self):
        self.driver.maximize_window()
        self.driver.set_page_load_timeout(self.page_load_timeout)
        self.driver.set_script_timeout(self.script_load_timeout)
        self.driver.implicitly_wait(self.implicitly_wait)

    def create_browser(self):
        self.set_driver()

    def switch_browser_by_index(self, index):
        self.driver = self.multi_driver[index]

